/*
 * Segment 2 (KEx),  SWE2.4 Email/Fax/SMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.emailfaxsms;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;

/**
 * Versandauftrag f&uuml;r E-Mails.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 */
public class EmailVersandAuftrag implements IVersandAuftrag {

	/**
	 * ID der Applikation.
	 */
	private long applikationsID;
	/**
	 * Die laufende Nummer des Auftrags.
	 */
	private long laufendeNummer;
	/**
	 * Der Hauptempf&auml;nger.
	 */
	private String empfaenger;
	/**
	 * Liste der Empf&auml;nger f&uuml;r eine Kopie.
	 */
	private final List<String> ccEmpfaenger = new LinkedList<String>();
	/**
	 * Liste der BCC-Empf&auml;nger.
	 */
	private final List<String> bccEmpfaenger = new LinkedList<String>();
	/**
	 * Der Absender.
	 */
	private String absender;
	/**
	 * Der Betreff.
	 */
	private String betreff;
	/**
	 * Text der Email.
	 */
	private String text;

	/**
	 * Getter f&uuml;r Absender.
	 *
	 * @return Absender
	 */
	public String getAbsender() {
		return absender;
	}

	@Override
	public long getAppID() {
		return applikationsID;
	}

	/**
	 * Getter f&uuml;r BCC-Empf&auml;nger.
	 *
	 * @return BCC-Empf&auml;nger
	 */
	public Collection<String> getBCCEmpfaenger() {
		return bccEmpfaenger;
	}

	/**
	 * Getter f&uuml;r Betreff.
	 *
	 * @return Betreff
	 */
	public String getBetreff() {
		return betreff;
	}

	/**
	 * Getter f&uuml;r CC-Empf&auml;nger.
	 *
	 * @return CC-Empf&auml;nger
	 */
	public Collection<String> getCCEmpfaenger() {
		return ccEmpfaenger;
	}

	/**
	 * Getter f&uuml;r Empf&auml;nger.
	 *
	 * @return Empf&auml;nger
	 */
	public String getEmpfaenger() {
		return empfaenger;
	}

	@Override
	public long getLaufNr() {
		return laufendeNummer;
	}

	/**
	 * Getter f&uuml;r Text der Mail.
	 *
	 * @return Text der Mail
	 */
	public String getText() {
		return text;
	}

	@Override
	public void init(final Data data) {

		applikationsID = data.getItem("MeldungsID").getItem("ApplikationsID").asUnscaledValue().longValue();
		laufendeNummer = data.getItem("MeldungsID").getItem("LaufendeNummer").asUnscaledValue().longValue();

		empfaenger = data.getItem("Empfaenger").asTextValue().getText();

		for (int i = 0; i < data.getItem("CCEmpfaenger").asTextArray().getLength(); i++) {
			final String s = data.getItem("CCEmpfaenger").asTextArray().getTextValue(i).getText();
			if (!s.equals("")) {
				ccEmpfaenger.add(s);
			}
		}

		for (int i = 0; i < data.getItem("BCCEmpfaenger").asTextArray().getLength(); i++) {
			final String s = data.getItem("BCCEmpfaenger").asTextArray().getTextValue(i).getText();
			if (!s.equals("")) {
				bccEmpfaenger.add(s);
			}
		}

		absender = data.getItem("Absender").asTextValue().getText();
		betreff = data.getItem("Betreff").asTextValue().getText();
		text = data.getItem("Text").asTextValue().getText();

	}

	@Override
	public String toString() {
		return "# Email Auftrag # AppID: " + String.valueOf(applikationsID) + " LaufendeNr: "
				+ String.valueOf(laufendeNummer) + " Absender: " + absender + " Empfaenger: " + empfaenger
				+ " CCEmpfaenger: " + ccEmpfaenger + " BCCEmpfaenger: " + bccEmpfaenger + " Betreff: " + betreff
				+ " Text: " + text;
	}
}
