/**
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:<br>
 * BitCtrl Systems GmbH<br>
 * Weißenfelser Straße 67<br>
 * 04229 Leipzig<br>
 * Phone: +49 341-490670<br>
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.emailfaxsms;

import java.util.Collection;

import org.junit.Test;

import de.bsvrz.kex.emailfaxsms.AsyncSerialPortCommunicator.ModemType;
import de.bsvrz.kex.emailfaxsms.sms.sender.KexSmsSender;
import de.bsvrz.kex.emailfaxsms.sms.sender.MapLatin1GSM;

/**
 * Testet den SMS Versand
 * 
 * @author BitCtrl Systems GmbH, Bachraty
 */
public class KexSmsSenderTester extends KexSmsSender {

	/**
	 * Parameter
	 */
	static final String TEST_EMPFAENGER = "+4915771531080";
	static final String TEST_PIN = "3311";
	static final String TEST_COM_PORT = "COM1";
	static final int TEST_BAUD_RATE = 57600;

	/**
	 * Standardkonstruktor
	 * 
	 * @throws Exception
	 */
	public KexSmsSenderTester() throws Exception {
		super(TEST_COM_PORT, TEST_BAUD_RATE, 30, TEST_PIN, ModemType.MODEM_GSM,
				"AT", "");
	}

	/**
	 * Konstruktor
	 * 
	 * @param com
	 *            COM-Port
	 * @param baud
	 *            Baudrate
	 * @param timeout
	 *            Timeout in Sekunden
	 * @throws Exception
	 *             Feher beim Zugriff zur serieller Schnittstelle
	 */
	public KexSmsSenderTester(String com, int baud, long timeout,
			String gsmPin, ModemType typ, String initSequenz, String prefix)
			throws Exception {
		super(com, baud, timeout, gsmPin, typ, initSequenz, prefix);
	}

	/**
	 * Die Testmethode
	 */
	@Test
	public void test() {
		try {
			KexSmsSenderTester sender = new KexSmsSenderTester();
			sender.testTextMode();
			testNachrichtTeilung();
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}

	/**
	 * Testet die Teilung der Nachrichten
	 * 
	 * @throws Exception
	 */
	public void testNachrichtTeilung() throws Exception {
		System.out.println("\nNachricht Zerbrechungstest:\n");
		String string = new String(
				"Test mit langer Text"
						+ "* Copyright (C) 2007 BitCtrl Systems GmbH"
						+ "* This program is free software; you can redistribute it and/or modify it under"
						+ "* the terms of the GNU General Public License as published by the Free Software"
						+ "* Foundation; either version 2 of the License, or (at your option) any later"
						+ "* version."
						+ "* This program is distributed in the hope that it will be useful, but WITHOUT"
						+ "* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS"
						+ "* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more"
						+ "* details.");

		Collection<String> teile = teileNachricht(string);
		for (String s : teile) {
			System.out.println("TEXT[" + s.length() + "]: " + s);
		}
	}

	/**
	 * Testet den Versand einer SMS
	 * 
	 * @throws Exception
	 */
	public void testTextMode() throws Exception {

		StringBuffer strBuff = new StringBuffer(160);
		String string = new String(
				"Test 160 Zeichen auch mit Umlauten ÜÖÄüöäßß_-.:;!\"§$%&/()=?<>@@@");
		strBuff.append(string);
		// 160 Zeichen
		for (int i = string.length(); i < 160; i++) {
			strBuff.append(i % 10);
		}

		string = strBuff.toString();
		string = MapLatin1GSM.mapTexModus(string);
		if (string.length() > 160) {
			System.err.println("Im Testfall nur 160 Zeichnen sollen versendet");
			return;
		}
		VersandStatus stat = send(TEST_EMPFAENGER, string, true);
		System.err.println("VERSAND STATUS: " + stat.getVersandFehlerMeldung());

	}
}
