/**
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:<br>
 * BitCtrl Systems GmbH<br>
 * Weißenfelser Straße 67<br>
 * 04229 Leipzig<br>
 * Phone: +49 341-490670<br>
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.emailfaxsms;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.emailfaxsms.AsyncSerialPortCommunicator.ModemType;
import de.bsvrz.kex.emailfaxsms.sms.sender.KexSmsSender;
import de.bsvrz.kex.emailfaxsms.sms.texter.KexSmsTexter;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;
import gnu.io.NoSuchPortException;

/**
 * Testet die Klasse KexSmsTexter
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class KexSmsTexterTest extends KexSmsTexter {

	/**
	 * Testet den Versand, stat Absenden macht er eine TextAusgabe
	 *
	 * @author BitCtrl Systems GmbH, Bachraty
	 *
	 */
	protected class KexSmsSenderTest extends KexSmsSender {

		/**
		 * Standardkonstruktor
		 *
		 * @param com
		 *            COM-Port
		 * @param baud
		 *            Baudrate
		 * @param timeout
		 *            Timeout in Sekunden
		 * @throws Exception
		 *             Feher beim Zugriff zur serieller Schnittstelle
		 */
		public KexSmsSenderTest(final String com, final int baud, final long timeout, final String gsmPin,
				final ModemType typ, final String initSequenz, final String prefix) throws Exception {
			super(com, baud, timeout, gsmPin, typ, initSequenz, prefix);
		}

		@Override
		public void sendInTextModus(final AsyncSerialPortCommunicator comm, final String empfaenger,
				final String textGSM, final String pin) throws CommunicationException {
			System.out.println("# SMS # EMPF: " + empfaenger + " TEXT:" + textGSM);
			try {
				Thread.sleep(1500);
			} catch (final Exception e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Konstruktor
	 *
	 * @param langeSMSzerbrechen
	 *            true, wenn lange SMS zerbrochen werden sollen
	 */
	public KexSmsTexterTest(final boolean langeSMSzerbrechen) {
		super(langeSMSzerbrechen);
	}

	@Override
	synchronized public void update(final ResultData[] results) {
		for (final ResultData resDatei : results) {
			final Data data = resDatei.getData();
			if (data == null) {
				continue;
			}
			if (resDatei.getDataDescription().getAttributeGroup().equals(ddParam.getAttributeGroup())) {

				final String com = data.getItem("SmsParameter").getItem("ModemParameter").getItem("COM-Port")
						.asTextValue().getText();
				final int baud = data.getItem("SmsParameter").getItem("ModemParameter").getItem("BaudRate")
						.asUnscaledValue().intValue();
				final long timeout = data.getItem("SmsParameter").getItem("ModemParameter").getItem("EmpfangsTimeout")
						.asUnscaledValue().longValue();

				final int typ = data.getItem("SmsParameter").getItem("ModemParameter").getItem("ModemTyp")
						.asUnscaledValue().intValue();

				final String gsmPin = data.getItem("SmsParameter").getItem("ModemParameter").getItem("GsmPin")
						.asTextValue().getText();
				final String init = data.getItem("SmsParameter").getItem("ModemParameter")
						.getItem("InitialisierungsBefehl").asTextValue().getText();
				final String prefix = data.getItem("SmsParameter").getItem("ModemParameter").getItem("WahlPraefix")
						.asTextValue().getText();

				stdEmpfaenger = data.getItem("StandardEmpfaenger").asTextValue().getText();

				ModemType modemTyp = ModemType.MODEM_FESTNETZ;
				switch (typ) {
				case 0:
					modemTyp = ModemType.MODEM_FESTNETZ;
					break;
				case 1:
					modemTyp = ModemType.MODEM_GSM;
					break;
				case 2:
					modemTyp = ModemType.MODEM_GPRS;
					break;
				}

				try {
					sender = new KexSmsSenderTest(com, baud, timeout, gsmPin, modemTyp, init, prefix);
				} catch (final NoSuchPortException e) {
					final String msg = "Fehler beim Starten von KexSmsSender: Die angegebene Schnittstelle existiert nicht: "
							+ com;
					getLogger().error(msg);
					MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.ERROR, msg);
				} catch (final Exception e) {
					final String msg = "Fehler beim Starten von KexSmsSender: " + e.getMessage();
					getLogger().warning(msg);
					MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.ERROR, msg);
				}
			}
		}
	}
}
