/*
 * Segment 2 (KEx),  SWE2.4 Email/Fax/SMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.emailfaxsms.fax.texter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.URL;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.emailfaxsms.AbstraktKexVersandKomponente;
import de.bsvrz.kex.emailfaxsms.AsyncSerialPortCommunicator.ModemType;
import de.bsvrz.kex.emailfaxsms.EmailFaxSmsMsg;
import de.bsvrz.kex.emailfaxsms.FaxVersandAuftrag;
import de.bsvrz.kex.emailfaxsms.IVersandAuftrag;
import de.bsvrz.kex.emailfaxsms.VersandStatus;
import de.bsvrz.kex.emailfaxsms.fax.sender.KexFaxSender;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;
import gnu.io.NoSuchPortException;

/**
 * Erzeugt Faxnachricht aus den eingegebenen Daten, die sie mit Hilfe der
 * KexFaxSender versendet.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 */
public class KexFaxTexter extends AbstraktKexVersandKomponente implements ClientReceiverInterface {

	/**
	 * Ghostscript-Executable Windows.
	 */
	private static final String GHOSTSCRIPT_WINDOWS_EXE = "gs.exe";

	/**
	 * Ghostscript Windows DLL.
	 */
	private static final String GHOSTSCRIPT_WINDOWS_DLL = "cygwin1.dll";

	/**
	 * Ghostscript-Executable Linux 64-Bit.
	 */
	private static final String GHOSTSCRIPT_LINUX64 = "gs64";

	/**
	 * Ghostscript-Executable Linux 32-Bit.
	 */
	private static final String GHOSTSCRIPT_LINUX32 = "gs32";

	/**
	 * Ghostscript Formatter.
	 */
	private static final String GHOSTSCRIPT_FORMAT = "gslp.ps";

	/**
	 * Ghostscript Fonts.
	 */
	private static final String GHOSTSCRIPT_FONTS = "fonts";

	/**
	 * Default Pfade zum Ghostscript.
	 */
	private String gsPATH;
	/**
	 * Default Pfade zum GSLP Script.
	 */
	private String gsLPPATH;
	/**
	 * Datenbeschreibung fuer die Parametrierung.
	 */
	protected DataDescription ddPARAM;
	/**
	 * StandardFaxKennung des Senders.
	 */
	protected String stdFaxKennung;
	/**
	 * StandardEmpfaenger.
	 */
	protected String stdEmpfaenger;
	/**
	 * Ob die Mitteilungen Hochaufloesend versendet werden.
	 */
	protected boolean stdHochaufloesend;
	/**
	 * SenderKlasse.
	 */
	protected KexFaxSender sender;

	/**
	 * Puffer fuer Fehlermeldungen.
	 */
	private final byte[] errorPuffer = new byte[4096];

	/**
	 * Konstruktor.
	 */
	public KexFaxTexter() {
		try {
			final File tmpDir = new File("~native");
			if (tmpDir.exists()) {

				File f;
				final String dir = tmpDir.getAbsolutePath();
				f = new File(dir + File.separator + KexFaxTexter.GHOSTSCRIPT_WINDOWS_EXE);
				if (f.exists() && !f.delete()) {
					throw new IOException("Die Datei: " + f.getAbsolutePath() + " ist entfernt worden");
				}

				f = new File(dir + File.separator + KexFaxTexter.GHOSTSCRIPT_LINUX32);
				if (f.exists() && !f.delete()) {
					throw new IOException("Die Datei: " + f.getAbsolutePath() + " ist entfernt worden");
				}

				f = new File(dir + File.separator + KexFaxTexter.GHOSTSCRIPT_LINUX64);
				if (f.exists() && !f.delete()) {
					throw new IOException("Die Datei: " + f.getAbsolutePath() + " ist entfernt worden");
				}

				f = new File(dir + File.separator + KexFaxTexter.GHOSTSCRIPT_FORMAT);
				if (f.exists() && !f.delete()) {
					throw new IOException("Die Datei: " + f.getAbsolutePath() + " ist entfernt worden");
				}

				f = new File(dir + File.separator + KexFaxTexter.GHOSTSCRIPT_WINDOWS_DLL);
				if (f.exists() && !f.delete()) {
					throw new IOException("Die Datei: " + f.getAbsolutePath() + " ist entfernt worden");
				}
			} else if (!tmpDir.mkdir()) {
				throw new IOException("Die Datei: " + tmpDir.getAbsolutePath() + " konnte nicht angelegt werden");
			}

			File gs;
			final File gslp = new File(tmpDir.getAbsolutePath() + File.separator + KexFaxTexter.GHOSTSCRIPT_FORMAT);
			extract(KexFaxTexter.GHOSTSCRIPT_FORMAT, gslp);
			gsLPPATH = gslp.getAbsolutePath();
			gslp.deleteOnExit();

			final String osName = System.getProperty("os.name");
			// WINDOWS
			if (osName.toLowerCase().indexOf("windows") != -1) {
				final File cygwin = new File(
						tmpDir.getAbsolutePath() + File.separator + KexFaxTexter.GHOSTSCRIPT_WINDOWS_DLL);
				extract(KexFaxTexter.GHOSTSCRIPT_WINDOWS_DLL, cygwin);
				gs = new File(tmpDir.getAbsolutePath() + File.separator + KexFaxTexter.GHOSTSCRIPT_WINDOWS_EXE);
				extract(KexFaxTexter.GHOSTSCRIPT_WINDOWS_EXE, gs);
				gsPATH = gs.getCanonicalPath();
				cygwin.deleteOnExit();
				gs.deleteOnExit();
			} else if (osName.toLowerCase().indexOf("linux") != -1) {
				final String arch = System.getProperty("os.arch");
				if (arch.equals("x86") || arch.equals("i386")) {
					gs = new File(tmpDir.getAbsolutePath() + File.separator + KexFaxTexter.GHOSTSCRIPT_LINUX32);
					extract(KexFaxTexter.GHOSTSCRIPT_LINUX32, gs);
					Runtime.getRuntime().exec("chmod +x " + gs.getAbsolutePath());
					gsPATH = gs.getAbsolutePath();
					gs.deleteOnExit();
				} else if (arch.equals("x86_64") || arch.equals("amd64")) {
					gs = new File(tmpDir.getAbsolutePath() + File.separator + KexFaxTexter.GHOSTSCRIPT_LINUX64);
					extract(KexFaxTexter.GHOSTSCRIPT_LINUX64, gs);
					Runtime.getRuntime().exec("chmod +x " + gs.getAbsolutePath());
					gsPATH = gs.getAbsolutePath();
					gs.deleteOnExit();
				} else {
					getLogger().error("Fehler beim Starten von KexFaxTexter: SystemArchitektur nicht unterstützt: " + arch);
					MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.ERROR,
							"Fehler beim Starten von KexFaxTexter: SystemArchitektur nicht unterstützt: " + arch);
					return;
				}
			} else {
				getLogger().error("Fehler beim Starten von KexfaxTexter: Betriebssystem nicht unterstützt: " + osName);
				MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.ERROR,
						"Fehler beim Starten von KexFaxTexter: Betriebsystem nicht unterstützt: " + osName);
				return;
			}
			tmpDir.deleteOnExit();
			// Warten bis die Files angelegt werden, sonst exec ergibt einen
			// Fehler
			Thread.sleep(1000);
			// Test, gibt nur die Version aus
			getLogger().config("GhostScript Kommando: \"" + gsPATH + "\"");
			final Process p = Runtime.getRuntime().exec(gsPATH + " -v");
			p.waitFor();
		} catch (final IOException e) {
			getLogger().warning("Fehler beim Starten von KexTexter: IOException: " + e.getMessage());
			MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.ERROR,
					"Fehler beim Starten von KexFaxTexter: IOException: " + e.getMessage());
		} catch (final InterruptedException e) {
			getLogger().warning("Fehler beim Starten von KexTexter: InterruptedException:" + e.getMessage());
			MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.ERROR,
					"Fehler beim Starten von KexFaxTexter: InterruptedException:" + e.getMessage());
		}
	}

	/**
	 * Extrahiert eine Ressource aus dem Jar ins temporaere File.
	 *
	 * @param resource
	 *            Resource Name
	 * @param file
	 *            temp File
	 * @throws IOException
	 *             Schreibfehler
	 */
	void extract(final String resource, final File file) throws IOException {

		if (resource == null) {
			throw new IOException("ressource == null");
		}

		if (file == null) {
			throw new IOException("file == null");
		}
		System.out.println("EXTRACT: " + resource + " File: " + file.getAbsolutePath());

		final OutputStream out = new FileOutputStream(file);

		final InputStream in = ClassLoader.getSystemResourceAsStream(resource);

		if (in == null) {
			out.close();
			throw new IOException("Die Ressource '" + resource + "' kann nicht gefunden werden.");
		}

		final byte[] buffer = new byte[4096];
		int n;
		while ((n = in.read(buffer)) > 0) {
			out.write(buffer, 0, n);
		}
		in.close();
		out.close();
	}

	/**
	 * Generiert eine Fax Mitteilung ins Temp File.
	 *
	 * @param absender
	 *            Absender
	 * @param hochaufloesend
	 *            true, wenn hochaufloesend
	 * @param auftrag
	 *            Versandauftrag
	 * @return File, in dem die Mitteilung als grafik gespeichert ist
	 * @throws IOException
	 *             Schreibfehler
	 * @throws InterruptedException
	 *             Generierung wurde unterbrochen
	 * @throws Exception
	 *             Ghostscript Prozess wurde nicht korrekt beendet
	 */
	protected File generiereFax(final String absender, final boolean hochaufloesend, final FaxVersandAuftrag auftrag)
			throws IOException, InterruptedException, Exception {

		final File outfile = File.createTempFile(Integer.toHexString((int) (Integer.MAX_VALUE * Math.random())), ".g3");

		final List<String> cmd = new LinkedList<String>();
		final StringBuffer s = new StringBuffer(256);
		cmd.add(gsPATH);

		s.append("-sDEVICE=");
		s.append(hochaufloesend ? "dfaxhigh" : "dfaxlow");
		cmd.add(s.toString());
		s.delete(0, s.length());

		s.append("-sOutputFile=");
		s.append(outfile.getCanonicalPath());
		cmd.add(s.toString());
		s.delete(0, s.length());

		cmd.add("-dNOPAUSE");
		cmd.add("-q");

		final URL url = ClassLoader.getSystemResource(KexFaxTexter.GHOSTSCRIPT_FONTS);
		if (url == null) {
			throw new IOException(
					"Das Verzeichnis '" + KexFaxTexter.GHOSTSCRIPT_FONTS + "' konnte nicht gefunden werden");
		}
		final File fdir = new File(url.toURI());

		cmd.add("-I" + fdir.getPath());

		cmd.add("--");
		cmd.add(gsLPPATH);

		cmd.add("-fCourier-Bold12");
		cmd.add("-B");
		cmd.add("%stdin");

		getLogger().fine("PROZESS STARTET: " + cmd);

		final String[] cmdarray = cmd.toArray(new String[1]);

		final Process p = Runtime.getRuntime().exec(cmdarray);
		final OutputStream out = p.getOutputStream();

		// TEST-Ausgaben
		final Thread debugreader = new Thread() {
			@Override
			public void run() {
				final BufferedReader r = new BufferedReader(new InputStreamReader(p.getInputStream()));
				String currentLine = null;
				try {
					while ((currentLine = r.readLine()) != null) {
						getLogger().info(currentLine);
					}
				} catch (final IOException e) {
					// e.printStackTrace();
					getLogger().info("Ghostscript IOException: " + e.getMessage());
				}
			}
		};

		debugreader.start();
		// Ende Testausgaben

		out.write("\t\t\t * FAX MITTEILUNG * \n".getBytes());
		out.write(String.format("VON: %s\n", absender).getBytes());
		int i = 0;
		for (final String str : auftrag.getEmpfaenger()) {
			if (i++ == 0) {
				out.write(String.format("AN:  %s\n", str).getBytes());
			} else {
				out.write(String.format("    %s\n", str).getBytes());
			}
		}

		out.write(String
				.format("\n\nApplikationsID: %d\nLaufende Nummer: %d\n\n", auftrag.getAppID(), auftrag.getLaufNr())
				.getBytes());
		out.write("---------------------------------------------------------------------------".getBytes());
		out.write(auftrag.getText().getBytes());
		out.close();

		final int ret = p.waitFor();
		getLogger().fine("PROZESS RETURN: " + ret);
		if (ret != 0) {
			int n = p.getErrorStream().read(errorPuffer);
			if (n < 0) {
				n = 0;
			}
			final String str = "Prozess beendet (" + ret + "): " + new String(errorPuffer, 0, n);
			throw new Exception(str);
		}
		return outfile;
	}

	@Override
	public void init(final ClientDavInterface connection, final SystemObject modul, final int maxWarteschlangeGroesse) {
		super.init(connection, modul, maxWarteschlangeGroesse);

		ddPARAM = new DataDescription(getDav().getDataModel().getAttributeGroup("atg.versandModulExterneMeldungenFax"),
				getDav().getDataModel().getAspect("asp.parameterSoll"));

		getDav().subscribeReceiver(this, modul, ddPARAM, ReceiveOptions.normal(), ReceiverRole.receiver());
		getLogger().fine("FaxTexter Initialisierung komplett");
	}

	@Override
	protected synchronized VersandStatus sendeAuftrag(final IVersandAuftrag auftrag, final int maxVersuche,
			final long pause) throws InterruptedException {

		// Warten auf die Parametrierung
		if (sender == null) {
			final String txt = MessageFormat.format(EmailFaxSmsMsg.WarningNochNichtParametriert.toString(),
					"FAX-Versand");
			LogTools.log(getLogger(), EmailFaxSmsMsg.WarningAllgemein, txt);
			return new VersandStatus(txt, false);
		}

		if (!(auftrag instanceof FaxVersandAuftrag)) {
			getLogger().warning("Fax-Versand Komponente kann nur Fax-Versand Aufträge bearbeiten");
			return VersandStatus.NICHT_VERSENDET;
		}

		final FaxVersandAuftrag eAuftr = (FaxVersandAuftrag) auftrag;
		getLogger().info("Der Fax-Auftrag: " + eAuftr + " wird bearbeitet");

		if ((!stdEmpfaenger.equals("")) && eAuftr.getEmpfaenger().isEmpty()) {
			eAuftr.addEmpfaenger(stdEmpfaenger);
		}
		if (eAuftr.getEmpfaenger().isEmpty() || (eAuftr.getText() == null)) {
			return new VersandStatus("Versandauftrag ist nicht komplett:" + eAuftr, false);
		}
		VersandStatus status = VersandStatus.NICHT_VERSENDET;

		File grafik;
		try {
			grafik = generiereFax(stdFaxKennung, stdHochaufloesend, eAuftr);
		} catch (final Exception e) {
			getLogger().warning("Fehler bei Erzeugung der Rastergrafik: " + e.getMessage());
			MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.WARNING,
					"KexFaxSender: Fehler bei Generierung der Rastergrafik: " + e.getMessage());
			return status;
		}
		int idx = 0;
		final Collection<String> empfaenger = eAuftr.getEmpfaenger();
		final ArrayList<VersandStatus> empfStatus = new ArrayList<VersandStatus>(empfaenger.size());
		for (int i = 0; i < empfaenger.size(); i++) {
			empfStatus.add(VersandStatus.NICHT_VERSENDET);
		}

		// fuer jeden Empfaenger maxVersuche
		for (final String empf : empfaenger) {
			for (int i = maxVersuche; i > 0; i--) {
				if (empf.equals("")) {
					empfStatus.set(idx, VersandStatus.NICHT_VERSENDET);
					break;
				}
				status = sender.send(stdFaxKennung, empf, stdHochaufloesend, grafik);
				empfStatus.set(idx, status);
				if (status.getVersandErfolg()) {
					break;
				}
				Thread.sleep(pause);
			}
			idx++;
		}
		grafik.delete();

		// Anzahl der Misserfolge
		idx = 0;
		for (final VersandStatus stat : empfStatus) {
			if (!stat.getVersandErfolg()) {
				idx++;
			}
		}

		if (idx > 0) {
			final StringBuffer str = new StringBuffer(512);
			str.append("Für folgende Empfänger konnte der Fax-Auftrag nicht versendet werden: ");
			int i = 0;
			for (final String empf : empfaenger) {
				if (empfStatus.get(i).getVersandErfolg()) {
					continue;
				}
				str.append(empf);
				str.append(" [");
				str.append(empfStatus.get(i).getVersandFehlerMeldung());
				str.append("], ");
				i++;
			}
			return new VersandStatus(str.toString(), false);
		}

		return VersandStatus.VERSAND_OK;
	}

	@Override
	public synchronized void update(final ResultData[] results) {
		for (final ResultData resDatei : results) {
			final Data data = resDatei.getData();
			if (data == null) {
				continue;
			}
			if (resDatei.getDataDescription().getAttributeGroup().equals(ddPARAM.getAttributeGroup())) {

				LogTools.log(getLogger(), EmailFaxSmsMsg.InfoParameterEmpfangen, "Fax");

				final String com = data.getItem("ModemParameter").getItem("COM-Port").asTextValue().getText();
				final int baud = data.getItem("ModemParameter").getItem("BaudRate").asUnscaledValue().intValue();
				final long timeout = data.getItem("ModemParameter").getItem("EmpfangsTimeout").asUnscaledValue()
						.longValue();

				final int typ = data.getItem("ModemParameter").getItem("ModemTyp").asUnscaledValue().intValue();

				final String gsmPin = data.getItem("ModemParameter").getItem("GsmPin").asTextValue().getText();
				String init = data.getItem("ModemParameter").getItem("InitialisierungsBefehl").asTextValue().getText();

				if ((init == null) || (init.length() == 0)) {
					init = "ATZ";
				}

				final String prefix = data.getItem("ModemParameter").getItem("WahlPraefix").asTextValue().getText();

				stdFaxKennung = data.getItem("FaxKennung").asTextValue().getText();
				final int hochaufloesend = data.getItem("Hochaufloesend").asUnscaledValue().intValue();
				if (hochaufloesend == 1) {
					stdHochaufloesend = true;
				} else {
					stdHochaufloesend = false;
				}

				stdEmpfaenger = data.getItem("StandardEmpfaenger").asTextValue().getText();

				ModemType modemTyp = ModemType.MODEM_FESTNETZ;
				switch (typ) {
				case 0:
					modemTyp = ModemType.MODEM_FESTNETZ;
					break;
				case 1:
					modemTyp = ModemType.MODEM_GSM;
					break;
				case 2:
					modemTyp = ModemType.MODEM_GPRS;
					break;
				default:
					getLogger().error("Modemtyp ungültig");
					return;
				}
				try {
					sender = new KexFaxSender(com, baud, timeout, modemTyp, gsmPin, init, prefix);
				} catch (final NoSuchPortException e) {
					getLogger().warning(
							"Fehler beim Starten von KexFaxSender: Die angegebene Schnittstelle existiert nicht: "
									+ com);
					MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.ERROR,
							"Fehler beim Starten von KexFaxSender: Die angegebene Schnittstelle existiert nicht: "
									+ com);
				} catch (final Exception e) {
					getLogger().warning("Fehler beim Starten von KexFaxSender: " + e.getMessage());
					MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.ERROR,
							"Fehler beim Starten von KexFaxSender: " + e.getMessage());
				}
			}
		}
	}
}
