/*
 * Segment 2 (KEx),  SWE2.4 Email/Fax/SMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.emailfaxsms.sms.texter;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.emailfaxsms.AbstraktKexVersandKomponente;
import de.bsvrz.kex.emailfaxsms.AsyncSerialPortCommunicator.ModemType;
import de.bsvrz.kex.emailfaxsms.EmailFaxSmsMsg;
import de.bsvrz.kex.emailfaxsms.IVersandAuftrag;
import de.bsvrz.kex.emailfaxsms.SmsVersandAuftrag;
import de.bsvrz.kex.emailfaxsms.VersandStatus;
import de.bsvrz.kex.emailfaxsms.sms.sender.KexSmsSender;
import de.bsvrz.kex.emailfaxsms.sms.sender.MapLatin1GSM;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;
import gnu.io.NoSuchPortException;

/**
 * Bearbeitet SMS-Versand Auftr&auml;ge, versendet sie mit der Hilfe der Klasse
 * KexSmsSender.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 */
public class KexSmsTexter extends AbstraktKexVersandKomponente implements ClientReceiverInterface {

	/**
	 * Datenbeschreibung fuer die Parametrierung.
	 */
	protected DataDescription ddParam;
	/**
	 * StandardEmpfaenger.
	 */
	protected String stdEmpfaenger;
	/**
	 * SenderKlasse.
	 */
	protected KexSmsSender sender;
	/**
	 * Ob lange SMS zerbrochen werden sollen.
	 */
	protected boolean langeSMSzerbrechen = false;

	/**
	 * Konstruktor.
	 *
	 * @param langeSMSzerbrechen
	 *            true, wenn lange SMS zerbrochen werden sollen
	 */
	public KexSmsTexter(final boolean langeSMSzerbrechen) {
		this.langeSMSzerbrechen = langeSMSzerbrechen;
	}

	@Override
	public void init(final ClientDavInterface connection, final SystemObject modul, final int maxWarteschlangeGroesse) {
		super.init(connection, modul, maxWarteschlangeGroesse);

		ddParam = new DataDescription(getDav().getDataModel().getAttributeGroup("atg.versandModulExterneMeldungenSms"),
				getDav().getDataModel().getAspect("asp.parameterSoll"));

		getDav().subscribeReceiver(this, modul, ddParam, ReceiveOptions.normal(), ReceiverRole.receiver());
		getLogger().fine("SmsTexter Initialisierung komplett");
	}

	@Override
	protected synchronized VersandStatus sendeAuftrag(final IVersandAuftrag auftrag, final int maxVersuche,
			final long pause) throws InterruptedException {

		// Warten auf die Parametrierung
		if (sender == null) {
			final String txt = MessageFormat.format(EmailFaxSmsMsg.WarningNochNichtParametriert.toString(),
					"SMS-Versand");
			LogTools.log(getLogger(), EmailFaxSmsMsg.WarningAllgemein, txt);
			return new VersandStatus(txt, false);
		}

		if (!(auftrag instanceof SmsVersandAuftrag)) {
			getLogger().warning("Sms-Versand Komponente kann nur Sms-Versand Aufträge bearbeiten");
			return VersandStatus.NICHT_VERSENDET;
		}

		final SmsVersandAuftrag eAuftr = (SmsVersandAuftrag) auftrag;

		final Collection<String> empfaenger = eAuftr.getEmpfaenger();
		if ((!stdEmpfaenger.equals("")) && empfaenger.isEmpty()) {
			empfaenger.add(stdEmpfaenger);
		}
		if (empfaenger.isEmpty() || (eAuftr.getText() == null) || eAuftr.getText().equals("")) {
			return new VersandStatus("Versandauftrag ist nicht komplett:" + eAuftr, false);
		}

		// Erstellen der Nachricht
		final String text = MapLatin1GSM.mapTexModus(eAuftr.getText());
		if ((!langeSMSzerbrechen) && (text.length() > KexSmsSender.GSM_TEXTMODE_SMS_LENGTH)) {
			getLogger().warning("SMS wurde nicht versendet, weil die max. Länge von 160 Zeichen überschritten wurde");
			MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.WARNING,
					"SMS wurde nicht versendet, weil die max. Länge 160 Zeichen überschritten wurde");
			return new VersandStatus("Nicht versendet - SMS zu lang", false);
		}

		int idx = 0;
		final ArrayList<VersandStatus> empfStatus = new ArrayList<VersandStatus>(empfaenger.size());
		for (int i = 0; i < empfaenger.size(); i++) {
			empfStatus.add(VersandStatus.NICHT_VERSENDET);
		}

		VersandStatus status = VersandStatus.NICHT_VERSENDET;

		// fuer jeden Empfaenger maxVersuche
		for (final String empf : empfaenger) {
			for (int i = maxVersuche; i > 0; i--) {
				if (empf.equals("")) {
					empfStatus.set(idx, VersandStatus.NICHT_VERSENDET);
					break;
				}
				status = sender.send(empf, text, false);
				empfStatus.set(idx, status);
				if (status.getVersandErfolg()) {
					break;
				}
				Thread.sleep(pause);
			}
			idx++;
		}

		// Anzahl der Misserfolge
		idx = 0;
		for (final VersandStatus stat : empfStatus) {
			if (!stat.getVersandErfolg()) {
				idx++;
			}
		}

		if (idx > 0) {
			final StringBuffer str = new StringBuffer(512);
			str.append("Fuer folgende Empfaenger konnte der SMS-Auftrag nicht versendet werden: ");
			int i = 0;
			for (final String empf : empfaenger) {
				if (empfStatus.get(i).getVersandErfolg()) {
					continue;
				}
				str.append(empf);
				str.append(" [");
				str.append(empfStatus.get(i).getVersandFehlerMeldung());
				str.append("], ");
				i++;
			}
			return new VersandStatus(str.toString(), false);
		}

		return VersandStatus.VERSAND_OK;
	}

	@Override
	public synchronized void update(final ResultData[] results) {
		for (final ResultData resDatei : results) {
			final Data data = resDatei.getData();
			if (data == null) {
				continue;
			}
			if (resDatei.getDataDescription().getAttributeGroup().equals(ddParam.getAttributeGroup())) {

				LogTools.log(getLogger(), EmailFaxSmsMsg.InfoParameterEmpfangen, "SMS");

				final String com = data.getItem("SmsParameter").getItem("ModemParameter").getItem("COM-Port")
						.asTextValue().getText();
				final int baud = data.getItem("SmsParameter").getItem("ModemParameter").getItem("BaudRate")
						.asUnscaledValue().intValue();
				final long timeout = data.getItem("SmsParameter").getItem("ModemParameter").getItem("EmpfangsTimeout")
						.asUnscaledValue().longValue();

				final int typ = data.getItem("SmsParameter").getItem("ModemParameter").getItem("ModemTyp")
						.asUnscaledValue().intValue();

				final String gsmPin = data.getItem("SmsParameter").getItem("ModemParameter").getItem("GsmPin")
						.asTextValue().getText();
				final String init = data.getItem("SmsParameter").getItem("ModemParameter")
						.getItem("InitialisierungsBefehl").asTextValue().getText();
				final String prefix = data.getItem("SmsParameter").getItem("ModemParameter").getItem("WahlPraefix")
						.asTextValue().getText();

				stdEmpfaenger = data.getItem("StandardEmpfaenger").asTextValue().getText();

				ModemType modemTyp = ModemType.MODEM_FESTNETZ;
				switch (typ) {
				case 0:
					modemTyp = ModemType.MODEM_FESTNETZ;
					break;
				case 1:
					modemTyp = ModemType.MODEM_GSM;
					break;
				case 2:
					modemTyp = ModemType.MODEM_GPRS;
					break;
				default:
					break;
				}

				try {
					sender = new KexSmsSender(com, baud, timeout, gsmPin, modemTyp, init, prefix);
				} catch (final NoSuchPortException e) {
					getLogger().warning(
							"Fehler beim Starten von KexSmsSender: Die angegebene Schnittstelle existiert nicht: "
									+ com);
					MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.ERROR,
							"Fehler beim Starten von KexSmsSender: Die angegebene Schnittstelle existiert nicht: "
									+ com);
				} catch (final Exception e) {
					getLogger().warning("Fehler beim Starten von KexSmsSender: " + e.getMessage());
					MessageSender.getInstance().sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.ERROR,
							"Fehler beim Starten von KexSmsSender: " + e.getMessage());
				}
			}
		}
	}

}
